#include <LedControl.h>
#include <Arduino.h>

// Pinii pentru LedControl
const int DIN_PIN = 11;
const int CLK_PIN = 13;
const int CS_PIN = 10;
const int BUZZER_PIN = 3;

// Pinii pentru joystick
const int VRxPin = A0;
const int VRyPin = A1;
const int SWPin = 2;

LedControl lc = LedControl(DIN_PIN, CLK_PIN, CS_PIN, 1);

// Structură pentru coordonate
struct Point {
  int x;
  int y;
};

// Lungimea maximă a șarpelui
const int maxLength = 6;
Point snake[maxLength];
int snakeLength = 2; // Inițializarea lungimii șarpelui la 2
Point fruit; 

// Direcția șarpelui
enum Direction { UP, DOWN, LEFT, RIGHT, NONE };
Direction dir = NONE; // Inițializare cu direcția NONE

// Variabile pentru starea jocului
bool gameStarted = false;
bool gameOver = false;
int fruitsEaten = 0; // Numarul de fructe mancate

void draw();
void moveSnake();
void readJoystick();
void checkCollision();
void generateFruit();
int smoothAnalogRead(int pin);
void displayGameOver();
void playBuzzerTone(int frequency, int duration);

void setup() {
  Serial.begin(9600);
  randomSeed(analogRead(0)); // Inițializare generator de numere aleatorii
  lc.shutdown(0, false); 
  lc.setIntensity(0, 8); // Setează intensitatea luminii (0-15)
  lc.clearDisplay(0);

  pinMode(VRxPin, INPUT);
  pinMode(VRyPin, INPUT);
  pinMode(SWPin, INPUT_PULLUP);
  pinMode(BUZZER_PIN, OUTPUT);

  // Inițializare șarpe 
  snake[0] = {3, 4};
  snake[1] = {4, 4};

  generateFruit();
  draw();
  playBuzzerTone(1000, 500);
}

void loop() {
  if (!gameOver) {
    // Citire joystick pentru schimbare direcție
    readJoystick();

    if (gameStarted) {
      // Mișcare șarpe
      moveSnake();
      // Verificare coliziune
      checkCollision();
      // Actualizare afișaj
      draw();
      delay(500);
    }
  } else {
    // Afișare mesaj de game over și clipire șarpe
    displayGameOver();
  }
}

void readJoystick() {
  int VRx = smoothAnalogRead(VRxPin);
  int VRy = smoothAnalogRead(VRyPin);

  // Adăugăm o zonă "moartă" în jurul centrului
  if (VRx < 300 && dir != RIGHT) {
    dir = LEFT;
    gameStarted = true;
  } else if (VRx > 600 && dir != LEFT) {
    dir = RIGHT;
    gameStarted = true;
  } else if (VRy < 300 && dir != DOWN) {
    dir = UP;
    gameStarted = true;
  } else if (VRy > 600 && dir != UP) {
    dir = DOWN;
    gameStarted = true;
  }
}

void moveSnake() {
  // Actualizare poziție șarpe doar dacă direcția nu este NONE
  if (dir != NONE) {
    // Mutare coadă
    for (int i = snakeLength - 1; i > 0; i--) {
      snake[i] = snake[i - 1];
    }

    // Mutare cap șarpe
    if (dir == LEFT) snake[0].x--;
    if (dir == RIGHT) snake[0].x++;
    if (dir == UP) snake[0].y--;
    if (dir == DOWN) snake[0].y++;
  }
}

void checkCollision() {
  // Verificare coliziune cu pereții
  if (snake[0].x < 0 || snake[0].x > 7 || snake[0].y < 0 || snake[0].y > 7) {
    gameOver = true;
    playBuzzerTone(1000, 1000); 
  }

  // Verificare coliziune cu corpul șarpelui
  for (int i = 1; i < snakeLength; i++) {
    if (snake[0].x == snake[i].x && snake[0].y == snake[i].y) {
      gameOver = true;
      playBuzzerTone(1000, 1000);
    }
  }

  // Verificare dacă șarpele a mâncat fructul
  if (snake[0].x == fruit.x && snake[0].y == fruit.y) {
    fruitsEaten++;
    lc.clearDisplay(0); 
    generateFruit(); 
    draw(); // Reactualizare afișaj după generarea fructului

    // Creștere lungime șarpe în funcție de numărul de fructe mâncate
    if ((snakeLength < maxLength) && ((snakeLength == 2 && fruitsEaten == 2) || 
        (snakeLength == 3 && fruitsEaten == 5) || 
        (snakeLength == 4 && fruitsEaten == 9) || 
        (snakeLength == 5 && fruitsEaten == 14))) {
      snakeLength++;
      playBuzzerTone(500, 200);
    }

    // Afișare scor periodic după ce a atins lungimea maximă
    if (snakeLength == maxLength || fruitsEaten % 3 == 0) {
      Serial.print("Fructe colectate: ");
      Serial.println(fruitsEaten);
    }
  }
}

void generateFruit() {
  bool valid = false;
  while (!valid) {
    fruit.x = random(0, 8);
    fruit.y = random(0, 8);
    valid = true;

    // Verificare să nu se suprapună cu șarpele
    for (int i = 0; i < snakeLength; i++) {
      if (fruit.x == snake[i].x && fruit.y == snake[i].y) {
        valid = false;
        break;
      }
    }
  }
}

void draw() {
  lc.clearDisplay(0); 
  delay(50); 
  
  // Desenare șarpe
  for (int i = 0; i < snakeLength; i++) {
    lc.setLed(0, snake[i].y, snake[i].x, true);
  }
  // Desenare fruct
  lc.setLed(0, fruit.y, fruit.x, true);
  
  
  Serial.print("Snake: ");
  for (int i = 0; i < snakeLength; i++) {
    Serial.print("(");
    Serial.print(snake[i].x);
    Serial.print(",");
    Serial.print(snake[i].y);
    Serial.print(") ");
  }
  Serial.println();
  
  Serial.print("Fruit: (");
  Serial.print(fruit.x);
  Serial.print(",");
  Serial.print(fruit.y);
  Serial.println(")");
}

void displayGameOver() {
  for (int i = 0; i < 5; i++) {
    lc.clearDisplay(0); 
    delay(500); 
    draw(); 
    delay(500); 
  }
  Serial.println("Game Over");
}

void playBuzzerTone(int frequency, int duration) {
  tone(BUZZER_PIN, frequency, duration);
  delay(duration);
  noTone(BUZZER_PIN);
}

int smoothAnalogRead(int pin) {
  const int numReadings = 10;
  int readings[numReadings];
  int total = 0;

  for (int i = 0; i < numReadings; i++) {
    readings[i] = analogRead(pin);
    total += readings[i];
    delay(10);  // Întârziere pentru a permite citirea stabilă
  }
  
  int average = total / numReadings;
  return average;
}
